/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule transfer-wizard-dialog-controller
 */

(function() {
var TransferWizardDialogController;
var prototype;
var superConstructor;
var TransferWizardSelectDstDialogView = dixim.contents.TransferWizardSelectDstDialogView;
var TransferWizardSelectStorageDialogView = dixim.contents.TransferWizardSelectStorageDialogView;
var TransferWizardTimingDialogView = dixim.contents.TransferWizardTimingDialogView;
var TransferWizardConfirmDialogView = dixim.contents.TransferWizardConfirmDialogView;
var TransferWizardStatusDialogView = dixim.contents.TransferWizardStatusDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var DeletionDialogController = dixim.contents.DeletionDialogController;
var Enum = dixim.ctt.util.Enum;
var encodeId = dixim.xml.encodeId;

/**
 * @class TransferWizardDialogController
 * @constructor
 * @param {Object} configs
 */
TransferWizardDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(TransferWizardDialogController, EventTarget);
dixim.contents.TransferWizardDialogController = TransferWizardDialogController;
prototype = TransferWizardDialogController.prototype;
superConstructor = TransferWizardDialogController.superConstructor;

var NEXT = 'next';
var UPDATE = 'update';
var FINISH = 'finish';
var START = 'start';
var CANCEL = 'cancel';
var CLOSE = 'close';
var CLEAR_SERVER = 'clearServer';
var SELECT_SERVER = 'selectServer';
var SELECT_STORAGE = 'selectStorage';

var Mode = Enum.fetch('DELETION');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    this._model = new ApiClientTransfer();
    this._model.addEventListener('fetchDmsList', this._successFetchDmsList, this);
    this._model.addEventListener('fetchDmsListError', this._errorFetchDmsList, this);
    this._model.addEventListener('removeDms', this._successRemoveDms, this);
    this._model.addEventListener('removeDmsError', this._errorRemoveDms, this);
    this._model.addEventListener('fetchDestStorageList', this._successFetchDestStorageList, this);
    this._model.addEventListener('fetchDestStorageListError', this._errorFetchDestStorageList, this);
    this._model.addEventListener('transfer', this._successTransfer, this);
    this._model.addEventListener('transferError', this._errorTransfer, this);
    this._model.addEventListener('cancelTransfer', this._successCancelTransfer, this);
    this._model.addEventListener('cancelTransferError', this._errorCancelTransfer, this);

    this._contentHeight = ['300', 'px'].join('');
    this._transferStatus = {};
    this._unknownContents = [];
    this._selectDstView;
    this._selectStorageView;
    this._timingView;
    this._confirmView;
    this._statusView;

    this._transferConfigs = {};
    this._transferConfigs.dataList = configs.dataList;
};

/**
 * @method start
 */
prototype.start = function() {
    this._model.addEventListener('fetchTransferStatus', this._successCheckFetchTransferStatus, this);
    this._model.addEventListener('fetchTransferStatusError', this._errorCheckFetchTransferStatus, this);

    this._fetchTransferStatus();
};

/**
 * @method stop
 */
prototype.stop = function() {
    if (typeof this._selectDstView !== 'undefined') this._selectDstView.hide();
    if (typeof this._selectStorageView !== 'undefined') this._selectStorageView.hide();
    if (typeof this._timingView !== 'undefined') this._timingView.hide();
    if (typeof this._confirmView !== 'undefined') this._confirmView.hide();
    if (typeof this._statusView !== 'undefined') this._statusView.hide();
};

prototype._successCheckFetchTransferStatus = function(e) {
    var query = {};
    var results = e.args.results;
    var resultsLen = results.length;
    var isTransferring = false;

    this._model.removeEventListener('fetchTransferStatus', this._successCheckFetchTransferStatus, this);
    this._model.removeEventListener('fetchTransferStatusError', this._errorCheckFetchTransferStatus, this);

    for (var i = 0; i < resultsLen; i += 1) {
        if (results[i].status === 'transferring') {
            isTransferring = true;
            break
        }
    }
    if (isTransferring) {
        this._transferConfigs.dataList = results;
        this._viewStatus(results);
    } else {
        this._viewSelectDst();
    }
};

prototype._errorCheckFetchTransferStatus = function() {
    this._model.removeEventListener('fetchTransferStatus', this._successCheckFetchTransferStatus, this);
    this._model.removeEventListener('fetchTransferStatusError', this._errorCheckFetchTransferStatus, this);
};

prototype._viewSelectDst = function() {
    var query = {};

    query.contentHeight = this._contentHeight;

    this._selectDstView = new TransferWizardSelectDstDialogView(query);
    this._selectDstView.addEventListener(CLEAR_SERVER, this._clickClearServer, this);
    this._selectDstView.addEventListener(SELECT_SERVER, this._clickSelectServer, this);
    this._selectDstView.addEventListener(CLOSE, this._returnMenu, this);
    this._selectDstView.show();
    this._selectDstView.startIndicator();
    this._model.fetchDmsList();
};

prototype._isSupportProfile = function(profiles, supportedProfiles) {
    for (var i = 0, ilen = profiles.length; i < ilen; i += 1) {
        var jlen = supportedProfiles.length;

        for (var j = 0; j < jlen; j += 1) {
            if (profiles[i] === supportedProfiles[j]) break;
        }
        if (j === jlen) return false;
    }
    return true;
};

prototype._successFetchDmsList = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;
    var contentsProfiles = [];
    var dataList = this._transferConfigs.dataList;
    var isNonAlive = false;
    var isSupported = false;

    for (var i = 0, len = dataList.length; i < len; i += 1) {
        contentsProfiles.push(dataList[i].profile);
    }
    contentsProfiles = contentsProfiles.filter(function(x, i, self) {
        return self.indexOf(x) === i;
    });

    for (var i = 0; i < resultsLen; i += 1) {
        if (this._isSupportProfile(contentsProfiles, results[i].supportedProfiles.split(','))) {
            this._selectDstView.addRow(results[i]);
            if (!!!results[i].alive) isNonAlive = true; 
            isSupported = true;
        }
    }

    if (!!!isSupported || (resultsLen === 0)) this._selectDstView.noRow();
    this._selectDstView.stopIndicator();
    this._selectDstView.changeOperationButton(isNonAlive);
};

prototype._errorFetchDmsList = function() {
    this._selectDstView.stopIndicator();
    this._selectDstView.error(_('contents_transfer_wizard_dst_fetch_error'));
};

prototype._clickClearServer = function(e) {
    this._removeDms(e.args);
};

prototype._removeDms = function(server_list) {
    var query = {};
    var controller;

    query.contents = server_list;
    query.mode = Mode.DMS;
    query.doubleDialog = true;

    controller = new DeletionDialogController(query);
    controller.addEventListener(UPDATE, this._reloadDmsList, this);
    controller.start();
};

prototype._reloadDmsList = function() {
    this._selectDstView.startIndicator();
    this._selectDstView.clearDmsList();
    this._model.fetchDmsList();
};

prototype._clickSelectServer = function(e) {
    this._transferConfigs.server = e.args.server;
    if (typeof this._transferConfigs.storage !== 'undefined') delete this._transferConfigs.storage;

    if (this._transferConfigs.server.hdlnkdoc === '') {
        this._viewTiming();
    } else {
        this._viewSelectStorage();
    }
};

prototype._viewSelectStorage = function() {
    var query = {};

    query.contentHeight = this._contentHeight;
    query.server = this._transferConfigs.server;

    this._selectStorageView = new TransferWizardSelectStorageDialogView(query);
    this._selectStorageView.addEventListener(SELECT_STORAGE, this._clickSelectStorage, this);
    this._selectStorageView.show();
    this._selectStorageView.startIndicator();

    query = {};
    query.udn = this._transferConfigs.server.udn;
    this._model.fetchDestStorageList(query);
};

prototype._successFetchDestStorageList = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;

    for (var i = 0; i < resultsLen; i += 1) {
        this._selectStorageView.addRow(results[i]);
    }
    this._selectStorageView.stopIndicator();
};

prototype._errorFetchDestStorageList = function(e) {
    this._selectStorageView.stopIndicator();
    this._selectStorageView.error(_('contents_transfer_wizard_dst_storage_fetch_error'));
};

prototype._clickSelectStorage = function(e) {
    this._transferConfigs.storage = e.args.storage;
    this._viewTiming();
};

prototype._viewTiming = function() {
    var query = {};

    query.contentHeight = this._contentHeight;

    this._timingView = new TransferWizardTimingDialogView(query);
    this._timingView.addEventListener(NEXT, this._clickNextTiming, this);
    this._timingView.show();
};

prototype._clickNextTiming = function(e) {
    this._transferConfigs.timing = e.args;

    this._viewConfirm();
};

prototype._viewConfirm = function() {
    var query = {};

    query.contentHeight = this._contentHeight;
    query.transferConfigs = this._transferConfigs;

    this._confirmView = new TransferWizardConfirmDialogView(query);
    this._confirmView.addEventListener(START, this._clickStartConfirm, this);
    this._confirmView.show();
};

prototype._clickStartConfirm = function(e) {
    this._transferConfigs.shutdown = e.args.shutdown;
    this._viewStatus();
};

prototype._viewStatus = function(results) {
    var query = {};

    query.contentHeight = this._contentHeight;
    if (typeof results !== 'undefined') {
        query.doubleDialog = true;
        query.srcType = results[0].src_type;
        query.dstType = results[0].dst_type;
    }
    this._model.addEventListener('fetchTransferStatus', this._successFetchTransferStatus, this);
    this._model.addEventListener('fetchTransferStatusError', this._errorFetchTransferStatus, this);

    this._statusView = new TransferWizardStatusDialogView(query);
    this._statusView.addEventListener(CANCEL, this._clickCancelStatus, this);
    this._statusView.addEventListener(CLOSE, this._clickClose, this);
    this._statusView.show();

    if (typeof results !== 'undefined') {
        this._successTransfer();
    } else {
        this._transfer();
    }
};

prototype._transfer = function() {
    var query = {};
    var dataList = this._transferConfigs.dataList;
    var dataListLen = dataList.length;
    var startTime = this._transferConfigs.timing.startTime;

    this._statusView.startIndicator();
    query.dst_udn = this._transferConfigs.server.udn;
    query.src_ids = [];
    for (var i = 0; i < dataListLen; i += 1) {
        query.src_ids.push(dataList[i].id);
    }
    if (typeof startTime !== 'undefined') query.start_time = startTime;
    if (typeof this._transferConfigs.storage !== 'undefined') query.dst_id = this._transferConfigs.storage.id;
    if (typeof this._transferConfigs.shutdown !== 'undefined') query.shutdown = this._transferConfigs.shutdown;
    this._model.transfer(query);
};

prototype._successTransfer = function(e) {
    var startTime;
    var dataList = this._transferConfigs.dataList;
    var dataListLen = dataList.length;

    this._statusView.stopIndicator();

    if (typeof this._transferConfigs.timing !== 'undefined') startTime = this._transferConfigs.timing.startTime;

    if (typeof startTime !== 'undefined') {
        this._statusView.setStatusDescription(_('contents_wizard_status_finish_reservation_transfer'), false);
        this._statusView.addReservationContents();
    } else {
        this._statusView.setStatusDescription(_('contents_wizard_status_transferring'), true);
        this.lastResults = $.merge([], dataList);
        for (var i = 0; i < dataListLen; i += 1) {
            this._statusView.addContent(encodeId(dataList[i].id), dataList[i].title);
        }
        this.status_timer = setInterval(dixim.proxy(this._fetchTransferStatus, this), 1000);
    }
};

prototype._errorTransfer = function(e) {
    var errorCode = e.args.errorCode;

    this._statusView.stopIndicator();
    switch(errorCode) {
        case 470:
            this._statusView.addReservationDuplicateErrorContents();
            break;
        default:
            this._statusView.addErrorContents();
            break;
    }
};

prototype._isUnknownValue = function(result) {
    if (typeof result.transferred !== 'number') return true;
    if (typeof result.total !== 'number') return true;
    if (result.total < 1) return true;
};

prototype._statusManager = function() {
    var STATUS = {
        UNKNOWN: 0x0000,
        ERROR: 0x0001,
        NOSPACE: 0x0002,
        UNSUPPORT: 0x0004,
        CANCEL: 0x0008,
        DONE: 0x0010,
    };
    var s = STATUS.UNKNOWN;;

    return {
        set: function(name) {
            s = s | STATUS[name];
        },
        unset: function(name) {
            s = (s & (~STATUS[name]));
        },
        isError: function() {
            if ((s & STATUS.ERROR) !== 0) return true;
            if ((s & STATUS.NOSPACE) !== 0) return true;
            if ((s & STATUS.UNSUPPORT) !== 0) return true;
            return false;
        },
        isMatch: function(name) {
            return ((s & STATUS[name]) !== 0);
        },
    };
};

prototype._successFetchTransferStatus = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;
    var percentage = 0;
    var cancelledStatusFlag = false;
    var isError = false;
    var numberOfError = 0;
    var statusManager = new this._statusManager();

    statusManager.set('DONE');

    for (var i = 0; i < resultsLen; i += 1) {
        isError = false;
        if (typeof this._unknownContents[i] === 'undefined') {
            this._unknownContents[i] = {};
        }
        if (results[i].status === 'error') {
            statusManager.set('ERROR');
            isError = true;
            this._transferStatus.isError = true;
        }
        if (results[i].status === 'errorNoSpace') {
            statusManager.set('NOSPACE');
            isError = true;
            this._transferStatus.isError = true;
        }
        if (results[i].status === 'errorUnsupported') {
            statusManager.set('UNSUPPORT');
            isError = true;
            this._transferStatus.isError = true;
        }
        if (isError) {
            this._statusView.errorProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''));
            numberOfError += 1;
            continue;
        }

        if (results[i].status !== 'done') statusManager.unset('DONE');
        if (results[i].status === 'cancelled') statusManager.set('CANCEL');

        if (this.timer === null) return;

        if (results[i].transferred && (! this._isUnknownValue(results[i]))) {
            if (results[i].transferred > results[i].total) {
                this._statusView.setRemainingTime(['#progress-bar-', encodeId(results[i].id)].join(''), 0);
                percentage = 100;
            } else {
                percentage = results[i].transferred / results[i].total * 100;
                if (typeof this.lastResults[i] === 'undefined' || typeof this.lastResults[i].transferTime === 'undefined') {
                    results[i].transferTime = 1;
                } else {
                    results[i].transferTime = this.lastResults[i].transferTime + 1;
                }
                results[i].transferSpeed = results[i].transferred / results[i].transferTime;
                this._statusView.setRemainingTime(['#progress-bar-', encodeId(results[i].id)].join(''), (results[i].total - results[i].transferred) / results[i].transferSpeed);
            }
            this._statusView.setProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''), percentage);
        }

        if (typeof results[i-1] !== 'undefined' && results[i-1].status !== 'done') {
            this._statusView.setProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''), 0);
            this._statusView.unknownProgressBar(['#progress-bar-', encodeId(results[i].id), ' #progress-description'].join(''));
            continue;
        }
        if (this._isUnknownValue(results[i])) {
            if (typeof this._unknownContents[i].transferred === 'undefined') {
                this._unknownContents[i].transferred = 1;
                this._unknownContents[i].total = 10;
            }
            this._unknownContents[i].transferred += 1;
            percentage = this._unknownContents[i].transferred / this._unknownContents[i].total * 100;
            this._statusView.setProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''), percentage);
            this._statusView.unknownProgressBar(['#progress-bar-', encodeId(results[i].id), ' #progress-description'].join(''));
        }
    }
    this.lastResults = results;

    if(statusManager.isMatch('DONE')) {
        this._finish();
    }
    if (statusManager.isMatch('CANCEL')) {
        this._successCancelTransfer();
    }
    if (numberOfError !== resultsLen) return;

    if (statusManager.isMatch('ERROR')) {
        this._statusView.setStatusDescription(_('contents_wizard_status_error'), false, true);
        return;
    }
    if (statusManager.isMatch('NOSPACE')) {
        this._statusView.setStatusDescription(_('contents_wizard_status_nospace_error'), false, true);
        return;
    }
    if (statusManager.isMatch('UNSUPPORT')) {
        this._statusView.setStatusDescription(_('contents_wizard_status_unsupported_error'), false, true);
        return;
    }
};

prototype._errorFetchTransferStatus = function(e) {
    if (typeof this._transferStatus.reached_count === 'undefined') this._transferStatus.errorReadchedCount = 0;
    this._transferStatus.errorReadchedCount += 1;

    if (this._transferStatus.errorReadchedCount === 10) {
        this._stopTransferStatus();
        this._statusView.setStatusDescription(_('contents_wizard_status_error'), false, true);
    }
};

prototype._finish = function() {
    if (this._transferStatus._error_flag) {
        this._statusView.setStatusDescription(_('contents_wizard_status_maybe_error'), true);
    } else {
        this._statusView.setStatusDescription(_('contents_wizard_status_finish_transfer'), false);
    }
    this._statusView.finished();
    this._stopTransferStatus();
    this.dispatchEvent(new Event(FINISH));
};

prototype._fetchTransferStatus = function() {
    this._model.fetchTransferStatus();
};

prototype._stopTransferStatus = function() {
    clearInterval(this.status_timer);
    this.status_timer = null;
};

prototype._clickCancelStatus = function() {
    this._statusView.startIndicator();
    this._model.cancelTransfer();
};

prototype._successCancelTransfer = function() {
    var resultsLen = this.lastResults.length;

    this._statusView.setStatusDescription(_('contents_wizard_status_cancel'));
    this._statusView.finished();
    this._stopTransferStatus();

    for (var i = 0; i < resultsLen; i += 1) {
        if (this.lastResults[i].status !== 'done') {
            this._statusView.errorProgressBar(['#progress-bar-', encodeId(this.lastResults[i].id)].join(''));
        }
    }
    this._statusView.stopIndicator();
};

prototype._errorCancelTransfer = function() {
    this._statusView.stopIndicator();
    this._statusView.setStatusDescription(_('contents_wizard_status_error'), false, true);
};

prototype._returnMenu = function() {
    this.dispatchEvent(new Event(CLOSE));
};

prototype._clickClose = function() {
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
};

}());
