/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule transfer-wizard-select-storage-dialog-view
 */

(function() {
var TransferWizardSelectStorageDialogView;
var prototype;
var superConstructor;
var ModalDialogView = dixim.common.ModalDialogView;
var Formatter = dixim.ctt.util.Formatter;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var ProgressBar = dixim.common.ProgressBar;
var $ = dixim.ctt.jQuery;

/**
 * @class TransferWizardSelectStorageDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
TransferWizardSelectStorageDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(TransferWizardSelectStorageDialogView, ModalDialogView);
dixim.contents.TransferWizardSelectStorageDialogView = TransferWizardSelectStorageDialogView;
superConstructor = TransferWizardSelectStorageDialogView.superConstructor;
prototype = TransferWizardSelectStorageDialogView.prototype;

var SELECT_STORAGE = 'selectStorage';
var TEMPLATE_BODY_CONTENT = [
    '<div class="description-field dlg_main_msg">',
    '<span>', '${description}', '</span>',
    '</div>',
    '<div id="storage-list" class="list-field list_next dlg_scroll">',
    '<table>', '</table>',
    '</div>',
].join('');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'transfer-wizard-select-storage-dialog-'].join('');

    configs.bodyContent = TEMPLATE_BODY_CONTENT
        .replace('${description}', _('contents_transfer_wizard_dst_storage_description'));

    configs.title = _('contents_operation_menu_transfer');
    configs.doubleDialog = true;
    configs.returnButton = dixim.proxy(this.hide, this);
    configs.contentHeight = configs.contentHeight || '300px';

    this._noActiveServerList = [];

    superConstructor.prototype._initialize.call(this, configs);
};

prototype.addRow = function(storage) {
    var bodySelector = $(['#body-content-', this._id].join(''));
    var tableSelector = bodySelector.find('#storage-list table');
    var rowSelector;
    var total;
    var available;
    var percentage;
    var progressBar = new ProgressBar();

    total = Formatter.stringifySize(storage.totalCapacity);
    available = Formatter.stringifySize(storage.availableCapacity);
    percentage = ((storage.totalCapacity - storage.availableCapacity) / storage.totalCapacity) * 100;

    tableSelector.append([
        '<tbody>',
        '<tr>',
        '<td>', '<span class="storage-name">', storage.friendlyName, '</span>', '</td>',
        '<td>',
        '<div name="progress">', '</div>',
        '<div class="capacity-cell list_detail txt_s">',
        '<p>', _('contents_transfer_wizard_dst_storage_capacity', available, total), '</p>',
        '</div>',
        '</td>',
        '</tr>',
        '</tbody>',
    ].join(''));

    rowSelector = tableSelector.find('tbody:last-child');

    progressBar.createProgressBar(rowSelector.find('[name="progress"]'));
    progressBar.setProgressBar(rowSelector.find('[name="progress"]'), percentage);

    rowSelector.on('click', storage, dixim.proxy(this._clickSelectStorage, this));
};

prototype._clickSelectStorage = function(e) {
    this.dispatchEvent(new Event(SELECT_STORAGE, {storage: e.data}));
};

prototype.error = function(text) {
    var bodySelector = $(['#body-content-', this._id].join(''));

    bodySelector.find('#storage-list').html([
        '<div class="txt_none error-text-field">',
        '<span>', text, '</span>',
        '</div>',
    ].join(''));
};

}());
