/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule language-view
 */

(function() {
var LanguageView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var UiUtils = dixim.common.UiUtils;
var $ = dixim.ctt.jQuery;

/**
 * @class LanguageView
 * @constructor
 * @param {Object} configs
 */
LanguageView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(LanguageView, EventTarget);
dixim.server.LanguageView = LanguageView;
prototype = LanguageView.prototype;
superConstructor = LanguageView.superConstructor;

var APPLY = 'apply';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._title = _('server_menu_language');
    this._uiUtils = UiUtils;
    this._languageList = ['ja', 'en', 'de', 'fr', 'it', 'es', 'nl', 'zh-cn', 'zh-tw', 'ko'];
};

/**
 * @method show
 */
prototype.show = function() {
    var languageListLen = this._languageList.length;

    $('.rootPage').html([
        '<div class="contents" data-enhance="false">',
        '<h2 id="tit_page">', this._title, '</h2>',
        '<div class="contents_inner">',
        '<div class="dl_tbl">',
        '<dl>',
        '<dt id="language_title">', _('server_language_title'), '</dt>',
        '<dd>',
        '<select id="language_val" class="select-pointer">', '</select>',
        '</dd>',
        '</dl>',
        '</div>',
        '</div>',
        '</div>',
    ].join(''));

    for (var i = 0; i < languageListLen; i += 1) {
        this._addLanguage({val: this._languageList[i], name: _(['server_language_list_', this._languageList[i]].join(''))});
    }

    this._uiUtils.cleanActionSettingBar();
    this._uiUtils.cleanActionBar();
    this._uiUtils.setActionBar('ab_apply', dixim.proxy(this._clickApply, this));

    $('.art_tit').text(this._title);
    $('.btn_prev').show().one('click', dixim.proxy(this._clickPrev, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
    $('.rootPage').empty();
};

prototype._addLanguage = function(data) {
    $('#language_val').append([
        '<option value="', data.val, '">', data.name, '</option>',
    ].join(''));
};

prototype.setLanguage = function(language) {
    var languageListLen = this._languageList.length;

    for (var i = 0; i < languageListLen; i += 1) {
        if (language === this._languageList[i]) {
            $('#language_val').val(language);
            break;
        }
    }
};

prototype.disableSetting = function() {
    $('#language_val').prop('disabled', true);
    this._uiUtils.disableActionBar('ab_apply');
};

prototype._clickApply = function() {
    this.dispatchEvent(new Event(APPLY, {language: $('#language_val').val()}));
};

prototype._clickPrev = function() {
    this.dispatchEvent(new Event(PREV));
};

}());
