/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule permission-menu-controller
 */

(function() {
var PermissionMenuController;
var prototype;
var superConstructor;
var PermissionMenuView = dixim.server.PermissionMenuView;
var EventTarget = dixim.event.EventTarget;
var PermissionOperationController = dixim.server.PermissionOperationController;
var PermissionDefaultSettingController = dixim.server.PermissionDefaultSettingController;

/**
 * @class PermissionMenuController
 * @constructor
 * @param {Object} configs
 */
PermissionMenuController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(PermissionMenuController, EventTarget);
dixim.server.PermissionMenuController = PermissionMenuController;
prototype = PermissionMenuController.prototype;
superConstructor = PermissionMenuController.superConstructor;

var ALLOW = 'allow';
var DENY = 'deny';
var DEFAULT = 'default';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._view = new PermissionMenuView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(ALLOW, this._clickAllow, this);
    this._view.addEventListener(DENY, this._clickDeny, this);
    this._view.addEventListener(DEFAULT, this._clickDefault, this);

    this._parentContext = configs.parentContext;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._clickAllow = function() {
    var query = {};
    var controller;

    query.parentContext = this;
    query.isAllowDevice = true;
    controller = new PermissionOperationController(query);
    controller.start();
};

prototype._clickDeny = function() {
    var query = {};
    var controller;

    query.parentContext = this;
    query.isAllowDevice = false;
    controller = new PermissionOperationController(query);
    controller.start();
};

prototype._clickDefault = function() {
    var query = {};
    var controller;

    query.parentContext = this;
    controller = new PermissionDefaultSettingController(query);
    controller.start()};

}());
