/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule server-menu-controller
 */

(function() {
var ServerMenuController;
var prototype;
var superConstructor;
var ServerMenuView = dixim.server.ServerMenuView;
var EventTarget = dixim.event.EventTarget;
var ServerBootController = dixim.server.ServerBootController;
var ActivationController = dixim.server.ActivationController;
var FriendlyNameController = dixim.server.FriendlyNameController;
var PermissionMenuController = dixim.server.PermissionMenuController;
var AutoDeleteController = dixim.server.AutoDeleteController;
var CompressSettingController = dixim.server.CompressSettingController;
var ExtensionsController = dixim.server.ExtensionsController;
var PublicFolderController = dixim.server.PublicFolderController;
var PairingController = dixim.server.PairingController;
var ParentalMenuController = dixim.server.ParentalMenuController;
var LanguageController = dixim.server.LanguageController;
var UiFunctionManager = dixim.ctt.util.UiFunctionManager;
var Configs = dixim.ctt.Configs;
var ResourceBundle = dixim.ctt.ResourceBundle;

/**
 * @class ServerMenuController
 * @constructor
 * @param {Object} configs
 */
ServerMenuController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ServerMenuController, EventTarget);
dixim.server.ServerMenuController = ServerMenuController;
prototype = ServerMenuController.prototype;
superConstructor = ServerMenuController.superConstructor;

UiFunctionManager.set('server');

var BOOT_SERVER = 'bootServer';
var ACTIVATION = 'activation';
var FRIENDLY_NAME = 'friendlyName';
var PERMISSION = 'permission';
var AUTO_DELETE = 'autoDelete';
var COMPRESS = 'compress';
var EXTENSIONS = 'extensions';
var PUBLIC_FOLDER = 'publicFolder';
var PAIRING = 'pairing';
var PARENTAL = 'parental';
var LANGUAGE = 'language';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    ResourceBundle.reset();
    Configs.set('resourceBundlePath', 'lang/server/');

    this._view = new ServerMenuView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(BOOT_SERVER, this._clickBootServer, this);
    this._view.addEventListener(ACTIVATION, this._clickActivation, this);
    this._view.addEventListener(FRIENDLY_NAME, this._clickFriendlyName, this);
    this._view.addEventListener(PERMISSION, this._clickPermission, this);
    this._view.addEventListener(AUTO_DELETE, this._clickAutoDelete, this);
    this._view.addEventListener(COMPRESS, this._clickCompress, this);
    this._view.addEventListener(EXTENSIONS, this._clickExtensions, this);
    this._view.addEventListener(PUBLIC_FOLDER, this._clickPublicFolder, this);
    this._view.addEventListener(PAIRING, this._clickPairing, this);
    this._view.addEventListener(PARENTAL, this._clickParental, this);
    this._view.addEventListener(LANGUAGE, this._clickLanguage, this);

    this._parentContext = configs.parentContext;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._clickBootServer = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new ServerBootController(query);
    controller.start();
};

prototype._clickActivation = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new ActivationController(query);
    controller.start();
};

prototype._clickFriendlyName = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new FriendlyNameController(query);
    controller.start();
};

prototype._clickPermission = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new PermissionMenuController(query);
    controller.start();
};

prototype._clickAutoDelete = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new AutoDeleteController(query);
    controller.start();
};

prototype._clickCompress = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new CompressSettingController(query);
    controller.start();
};

prototype._clickExtensions = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new ExtensionsController(query);
    controller.start();
};

prototype._clickPublicFolder = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new PublicFolderController(query);
    controller.start();
};

prototype._clickPairing = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new PairingController(query);
    controller.start();
};

prototype._clickParental = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new ParentalMenuController(query);
    controller.start();
};

prototype._clickLanguage = function() {
    var controller;
    var query = {};

    query.parentContext = this;
    controller = new LanguageController(query);
    controller.start();
};

}());
