/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule server-menu-view
 */

(function() {
var ServerMenuView;
var prototype;
var superConstructor;
var MenuView = dixim.common.MenuView;
var _ = dixim.ctt.ResourceBundle.get;
var UiFunctionManager = dixim.ctt.util.UiFunctionManager;

/**
 * @class ServerMenuView
 * @constructor
 * @param {Object} configs
 */
ServerMenuView = function(configs) {
    MenuView.call(this, configs);
};

dixim.inherit(ServerMenuView, MenuView);
dixim.server.ServerMenuView = ServerMenuView;
prototype = ServerMenuView.prototype;
superConstructor = ServerMenuView.superConstructor;

var BOOT_SERVER = 'bootServer';
var ACTIVATION = 'activation';
var FRIENDLY_NAME = 'friendlyName';
var PERMISSION = 'permission';
var AUTO_DELETE = 'autoDelete';
var COMPRESS = 'compress';
var EXTENSIONS = 'extensions';
var PUBLIC_FOLDER = 'publicFolder';
var PAIRING = 'pairing';
var PARENTAL = 'parental';
var LANGUAGE = 'language';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.title = _('common_menu_title_server');
    configs.menuList = [];

    configs.menuList.push({
        strId: 'server_menu_boot_server', eventParam: BOOT_SERVER,
    });

    if (UiFunctionManager.isImplement('dtcp_activation')) {
        configs.menuList.push({
            strId: 'server_menu_activation', eventParam: ACTIVATION,
        });
    }

    configs.menuList.push({
        strId: 'server_menu_friendly_name', eventParam: FRIENDLY_NAME,
    });

    configs.menuList.push({
        strId: 'server_menu_permission', eventParam: PERMISSION,
    });

    if (UiFunctionManager.isImplement('autoDelete')) {
        configs.menuList.push({
            strId: 'server_menu_auto_delete', eventParam: AUTO_DELETE,
        });
    }

    if (UiFunctionManager.isImplement('compress')) {
        configs.menuList.push({
            strId: 'server_menu_compress', eventParam: COMPRESS,
        });
    }

    configs.menuList.push({
        strId: 'server_menu_extensions', eventParam: EXTENSIONS,
    });

    configs.menuList.push({
        strId: 'server_menu_public_folder', eventParam: PUBLIC_FOLDER,
    });

    configs.menuList.push({
        strId: 'server_menu_pairing', eventParam: PAIRING,
    });

    configs.menuList.push({
        strId: 'server_menu_parental', eventParam: PARENTAL,
    });

    configs.menuList.push({
        strId: 'server_menu_language', eventParam: LANGUAGE,
    });

    superConstructor.prototype._initialize.call(this, configs);
};

}());
