/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule select-howto-download-dialog-controller
 */

(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var SelectHowtoDownloadDialogView = dixim.ctt.ad.SelectHowtoDownloadDialogView;
var FilterSettingDialogEditController = dixim.ctt.ad.FilterSettingDialogEditController;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;
var ErrorDialog = dixim.ctt.ErrorDialog;

var SelectHowtoDownloadDialogController;
var prototype;

/**
 * @class SelectHowtoDownloadDialogController
 * @constructor
 * @param {Object} configs
 * @param {AutoDownloadApiClient} [configs.model]
 */
SelectHowtoDownloadDialogController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(SelectHowtoDownloadDialogController, EventTarget);
dixim.ctt.ad.SelectHowtoDownloadDialogController = SelectHowtoDownloadDialogController;
prototype = SelectHowtoDownloadDialogController.prototype;

/**
 * Triggered when the deletion has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.deleted true if the deletion is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {AutoDownloadApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = new AutoDownloadApiClient();
  this._view = new SelectHowtoDownloadDialogView(configs);
  this._session = null;
  this._queue = [];
  this._finish = configs.finish || function() {return;};
  this._back = configs.back || function() {return;};
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;

  v.addEventListener('submit', this._didClickSubmit, this);
  v.addEventListener('cancel', this._didClickCancel, this);
  v.addEventListener('back', this._didClickBack, this);
  v.addEventListener('edit', this._didClickEdit, this);
  v.addEventListener('delete', this._didClickDelete, this);
  v.addEventListener('allselect', this._didClickAllSelect, this);
  v.addEventListener('allunselect', this._didClickAllUnselect, this);


  v.show();
};

prototype._didClickSubmit = function() {
  var select_info  = this._view.getSelectInfo();
  var filter_list = [];

  $('#select_filter_list_table [type="checkbox"]:checked').each(function() {
    filter_list.push($(this).next().val());
  });
  select_info.filterList = filter_list;
  this.destroy();
  this._finish(select_info);
};

prototype._didClickCancel = function() {
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE));
};

prototype._didClickBack = function() {
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE));
  this._back();
};

prototype._didClickEdit = function() {
  var query = {};
  var edit_dialog;
  var checkAndChangeName = function(name) {
    var new_name = '';
    var name_length = 0;
    var name_array = name.split('');
    var check_length = function(c) {
      if (c.match(/[a-z0-9]/)) {
        return 1;
      } else {
        return 1.8;
      }
    };

    for (var i = 0; i < name_array.length; i += 1) {
      name_length += check_length(name_array[i]);
      if (name_length < 55) {
        new_name += name_array[i];
      } else {
        new_name += "...";
        break;
      }
    }
    return new_name;
  };

  if ($('#select_filter_list_table [type="checkbox"]:checked').length === 1) {
    query.filterId = $('#select_filter_list_table [type="checkbox"]:checked').next().val();
    query.afterClose = dixim.proxy(function(data) {
      if (typeof data.filterName !== "undefined") {
        $('option').each(function() {
           if ($(this).val() == data.filterId) {
             $(this).html(checkAndChangeName(data.filterName));
            }
         });
       }
    }, this);
    edit_dialog = new FilterSettingDialogEditController(query);
    edit_dialog.start();
  } else {
    this._showErrorDialog('error_dialog_does_not_change_one_filter');
  }
};

prototype._didClickDelete = function() {
  $('#select_filter_list_table [type="checkbox"]:checked').closest('tr').remove();
};

prototype._didClickAllSelect = function() {
  $('#select_filter_list_table [type="checkbox"]').not(":last").prop("checked", true).trigger('change');
};

prototype._didClickAllUnselect = function() {
  $('#select_filter_list_table [type="checkbox"]').prop("checked", false).trigger('change');
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
