/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule select-src-container-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;
var ErrorDialog = dixim.ctt.ErrorDialog;

var SelectSrcContainerDialogView;
var prototype;
var superConstructor;

/**
 * @class SelectSrcContainerDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
SelectSrcContainerDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(SelectSrcContainerDialogView, ModalDialogView);
dixim.ctt.ad.SelectSrcContainerDialogView = SelectSrcContainerDialogView;
prototype = SelectSrcContainerDialogView.prototype;
superConstructor = SelectSrcContainerDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div id="select-src-container"/>';
var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

var TEMPLATE_BODY_CONTENT = ['<h1 id="task-add-h1">',
                             '</h1>',
                             '<div id="auto-download-tool-controller-server">',
                             '</div>'
                             ].join('');
var NODE_ID_PREFIX = 'src-container-id-';

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the next button was clicked.
 *
 * @event next
 */
var NEXT = 'next';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var buttonsError;

  configs  = configs || {};

  buttons = [
    {
      'text': _('ctt_transfer_wizard_button_label_next'),
      id: 'next',
      click: $.proxy(this._didClickNext, this),
      align: 'right',
      disabled: true
    },
    {
      'text': _('ctt_dialog_button_label_cancel'),
      id: 'cancel',
      click: $.proxy(this._didClickCancel, this),
      align: 'left'
    }
  ];

  buttonsError = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  configs.title = configs.title || _('ctt_ad_wizard_select_src_container');
  configs.contentBox = TEMPLATE_CONTENT_BOX;
  configs.bodyContent = TEMPLATE_BODY_CONTENT;
  configs.buttons = buttons;
  configs.width = 600;
  configs.height = 580;

  this._model = new AutoDownloadApiClient();
  this._loadingIcon = $(TEMPLATE_LOADING);
  this._errorNode = null;
  this._errorButtons = buttonsError;
  this.result = {};
  this.checkObjectId = new this._didObjectId();

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didSelectOneCheckbox = function() {
  var length = $('#auto-download-tool-controller-server-list input:checked').length;

  if (length !== 1) {
    this.disableButton($('#next'));
  } else {
    this.enableButton($('#next'));
  }
};

prototype._didHistoryHeader = function(str ,isHistoryHeader) {
  if (isHistoryHeader) {
    $('.history_header').show().html(str);
    $('.history_list').empty().hide();
  } else {
    $('.history_header').empty().hide();
    $('.history_list').show().html(str);
  }
};

prototype._didAddSrcServerListRow = function(device) {
  var icon = (device.icons && device.icons[0]) ? device.icons[0].url: "./img/default_dms_sm.jpg";
  var udn = device.udn.split(':')[1];
  var friendlyName = device.friendlyName.replace(/\s+/g, "");

  $('#auto-download-tool-controller-server-list').append(
     [
      '<tr>',
      '<td>',
      '<img width="45" height="45" class=',udn,' name="icon" value=',friendlyName,' src=',icon,'>',
      '<input type="button" class=',udn,' name="friendlyName" value=',friendlyName,'>',
      '</td>',
      '</tr>'
    ].join('')
  );

  $(['.',udn].join('')).one('click', dixim.proxy(function() {
    this.result.srcUdn = ['uuid:', udn].join('');
    this.result.friendlyName = device.friendlyName;
    this._didCreateSrcRootContainerTable();
  }, this));
};

prototype._didCreateSrcServerListTable = function() {
  var fetch_source_device_list_query = {};

  $('#task-add-h1').html(
     [
      '<span class="history_header"></span>',
      '<span class="history_list"></span>'
     ].join('')
   );
  $('#auto-download-tool-controller-server').html(
    [
      '<table id="auto-download-tool-controller-server-list" class="server-contents-list"></table>'
    ].join('')
  );

  this._didHistoryHeader(_("ctt_ad_wizard_src_servar_list_str"), true);

  fetch_source_device_list_query.success = dixim.proxy(function(e) {
    var devices = e.args.sources;
    var devices_length = devices.length;

    for (var i = 0; i < devices_length; i += 1) {
      this._didAddSrcServerListRow(devices[i]);
    }
  }, this);
  fetch_source_device_list_query.complete = dixim.proxy(function() {
    this._didSelectOneCheckbox();
  }, this);

  this._model.fetchSourceDeviceList(fetch_source_device_list_query);
};

prototype._didAddParentSrcContainerRow = function() {
  $('#auto-download-tool-controller-server-list').html(
     [
      '<tr>',
      '<td>',
      '<img src="./img/ico_dir.png" width="35" height="25" class="dir_icon parent_container" name="parent_src_icon">',
      '<input type="button" id="parent_src_container_btn" class="parent_container" value=',_("ctt_ad_wizard_return_parent_container"),'>',
      '</td>',
      '</tr>'
    ].join('')
  );
};

prototype._didAddRootContainerRow = function() {
  $('#auto-download-tool-controller-server-list').append(
    [
      '<tr>',
      '<td>',
      '<input type="checkbox" class="',[NODE_ID_PREFIX, '0'].join(''),'" name="chkbox_adt">',
      '<input type="button" class="root_class ',[NODE_ID_PREFIX, '0'].join(''),'" value=',_("ctt_ad_wizard_root_container"),'>',
      '</td>',
      '</tr>'
    ].join('')
  );
  $('.root_class').one('click', dixim.proxy(function() {
    this._didPushHistory($('.root_class').val());
    this.checkObjectId.setCurrentObjectId("0");
    this._didCreateSrcContainerTable("0");
    $('.parent_container').unbind('click');
    $('[type="checkbox"]').off('click');
  }, this));
  $('[type="checkbox"]').on('click', dixim.proxy(function() {
    this._didSelectOneCheckbox();
  }, this));
};

prototype._didCreateSrcRootContainerTable = function() {
  this._didHistoryHeader(this.result.friendlyName, false);
  $('#auto-download-tool-controller-server-list').empty();
  this._didAddParentSrcContainerRow();
  this._didAddRootContainerRow();
  this._didSelectOneCheckbox();
  $('.parent_container').one('click', dixim.proxy(function() {
    this._didCreateSrcServerListTable();
    $('.root_class').unbind('click');
    $('[type="checkbox"]').off('click');
  }, this));
};

prototype._didPushHistory = function(title) {
  var history = $('.history_list').text();

  if (history) history += " > ";
  history += title;
  this._didHistoryHeader(history, false);
};

prototype._didPopHistory = function() {
  var history = $(".history_list").text();
  var histryArray = history.split(" > ");
  var new_history = "";
  var histryArray_length = histryArray.length - 1;

  for (var i = 0; i < histryArray_length; i += 1) {
    if (i > 0) new_history += " > ";
    new_history += histryArray[i];
  }
  this._didHistoryHeader(new_history, false);
  return histryArray[histryArray.length - 1];

};

prototype._didObjectId = function() {
  var currentObjectIds = ["none"];

  return {
    setCurrentObjectId: function(id) {
      currentObjectIds.push(id);
    },
    getCurrentObjectId: function() {
      return currentObjectIds[currentObjectIds.length - 1];
    },
    getParentObjectId: function() {
      return currentObjectIds[currentObjectIds.length - 2];
    },
    popObjectIds: function() {
      return currentObjectIds.pop();
    }
  };
};

prototype._didEncodeString = function(str) {
  var enc_str;

  enc_str = dixim.xml.encodeId(str);
  enc_str = [NODE_ID_PREFIX, enc_str].join('');
  return enc_str;
};

prototype._didDecodeString = function(str) {
  var dec_str;

  dec_str = str.replace(new RegExp('^' + NODE_ID_PREFIX), '');
  if (dec_str !== '0') {
    dec_str = dixim.xml.decodeId(dec_str);
  }
  return dec_str;
};

prototype._didAddSrcContainerRow = function(containerInfo) {
  var icon = "";
  var button = "";
  var esc_objectId = containerInfo.objectId;
  var title = containerInfo.title;

  esc_objectId = this._didEncodeString(esc_objectId);
  if (containerInfo['class'].indexOf("object.container") !== -1) {
    icon = ['<input type="checkbox" class=',esc_objectId,' name="checkbox_adt">'].join('');
    button = ['<input type="button" class="src_container_class ',esc_objectId,'" value=', title,'>'].join('');
  } else if (containerInfo['class'].indexOf("object.item.videoItem") !== -1) {
    icon = '<img src="./img/ico_video.png" width="35" height="25" name="item_icon">';
    button = ['<span id=',esc_objectId,' class="item_title_button">',title,'</span>'].join('');
  } else if (containerInfo['class'].indexOf("object.item.audioItem") !== -1) {
    icon = '<img src="./img/ico_music.png" width="35" height="25" name="item_icon">';
    button = ['<span id=',esc_objectId,' class="item_title_button">',title,'</span>'].join('');
  } else if (containerInfo['class'].indexOf("object.item.imageItem") !== -1) {
    icon = '<img src="./img/ico_photo.png" width="35" height="25" name="item_icon">';
    button = ['<span id=',esc_objectId,' class="item_title_button">',title,'</span>'].join('');
  }

  $('#auto-download-tool-controller-server-list').append(
    [
     '<tr><td>',
     icon,
     button,
     '</td></tr>'
    ].join('')
  );

  $(['.',esc_objectId,'.src_container_class'].join('')).one('click', dixim.proxy(function() {
    var objectId;

    objectId = this._didDecodeString(esc_objectId);
    this._didPushHistory(title);
    this.checkObjectId.setCurrentObjectId(objectId);
    this._didCreateSrcContainerTable(objectId);
    $('[type="checkbox"]').off('click');
  }, this));
  $('[type="checkbox"]').on('click', dixim.proxy(function() {
    this._didSelectOneCheckbox();
  }, this));
};

prototype._didCreateSrcContainerTable = function(containerId) {
  var browse_query = {
      udn: this.result.srcUdn,
      objectId: containerId,
      browseFlag: "BrowseDirectChildren"
  };
  var parentObjectId = this.checkObjectId.getParentObjectId();

  browse_query.success = dixim.proxy(function(e) {
    var containers_info = e.args.objects;
    var containers_info_length = containers_info.length;

    $('#auto-download-tool-controller-server-list').empty();
    this._didAddParentSrcContainerRow();

    $('.parent_container').one('click', dixim.proxy(function() {
      var parent_container_name = this._didPopHistory();
      $('[type="checkbox"]').off('click');

      this.checkObjectId.popObjectIds();
      if (parent_container_name === _("ctt_ad_wizard_root_container")) {
        this._didCreateSrcRootContainerTable();
      } else {
        this._didCreateSrcContainerTable(parentObjectId);
       }
    }, this));

    for (var i = 0; i < containers_info_length; i += 1) {
      this._didAddSrcContainerRow(containers_info[i]);
    }
  }, this);
  browse_query.error = dixim.proxy(function() {
    this._showErrorDialog("ctt_error_dialog_default");
    this._didCreateSrcRootContainerTable();
  }, this);
  browse_query.complete = dixim.proxy(function() {
    this._loadingIcon.detach();
    $('[type="button"]').prop('disabled', false);
    $('.parent_container').bind();
    this._didSelectOneCheckbox();
  }, this);
  $('.ui-dialog-buttonset').append(this._loadingIcon);
  $('[type="button"]').prop('disabled', true);
  $('.parent_container').unbind();

  this._model.browse(browse_query);
};

prototype._didRenderDialog = function(e) {
  $('#select-src-container').css('overflow', 'hidden');
  this._didCreateSrcServerListTable();
  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickCancel = function() {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

prototype._didClickNext = function() {
  var objectId = $('#auto-download-tool-controller-server-list input:checked').prop('class');

  objectId = this._didDecodeString(objectId);
  this.result.srcContainerId = objectId;
  this.dispatchEvent(new Event(NEXT));
};

prototype.getSelectInfo = function() {
  return this.result;
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
