/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule confirmation-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var TickerView = dixim.ctt.TickerView;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var ConfirmationDialogView;
var prototype;
var superConstructor;

/**
 * @class ConfirmationDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
ConfirmationDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(ConfirmationDialogView, ModalDialogView);
dixim.ctt.ad.ConfirmationDialogView = ConfirmationDialogView;
prototype = ConfirmationDialogView.prototype;
superConstructor = ConfirmationDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-deletion-view-wrapper" />';

var TEMPLATE_BODY_CONTENT = ['<div class="ctt-deletion-view">',
                               '<div class="ctt-deletion-view-description"></div>',
                               '<div class="ctt-deletion-view-error"></div>',
                               '<div class="ctt-deletion-view-progress">',
                               '</div>',
                             '</div>'].join('');

var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the delete button was clicked.
 *
 * @event delete
 */
var DELETE = 'delete';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var buttonsError;
  var bodyContent;
  var yes_button;

  configs  = configs || {};

  if (configs.isDelete) {
    yes_button = _('ctt_deletion_view_button_label_delete');
  } else {
    yes_button = _('ctt_compression_confirmation_button_ok');
  }
  buttons = [
    {
      'text': yes_button,
      click: $.proxy(this._didClickDelete, this)
    },
    {
      'text': _('ctt_dialog_button_label_cancel'),
      click: $.proxy(this._didClickCancel, this),
      align: 'left'
    }
  ];

  buttonsError = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  bodyContent = TEMPLATE_BODY_CONTENT;

  configs.title = configs.title || _('ctt_deletion_view_title');
  configs.contentBox = configs.contentBox || TEMPLATE_CONTENT_BOX;
  configs.bodyContent = configs.bodyContent || bodyContent;
  configs.buttons = buttons;
  configs.width = 480;
  configs.height = 'auto';
  configs.maxHeight = 340;

  this._descriptionNode = null;
  this._errorNode = null;
  this._progressNode = null;
  this._ticker = null;
  this._targetItems = configs.targetItems;
  this._loadingIcon = null;
  this._inProgress = false;
  this._errorButtons = buttonsError;
  this._descriptionString = configs.description || _('ctt_ad_confirmation_deletion_cond_dialog_description');
  this._inProgressString = configs.in_progress || _('ctt_ad_confirmation_deletion_cond_dialog_description_in_progress');

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didRenderDialog = function(e) {
  var v = this._view;

  this._descriptionNode = $('.ctt-deletion-view-description', v).html(this._descriptionString);
  this._progressNode = $('.ctt-deletion-view-progress', v);
  this._ticker = new TickerView({
    containerNode: $('.ctt-deletion-view-error', v),
    type: 'error'
  });
  this._loadingIcon = $(TEMPLATE_LOADING);

  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickCancel = function() {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

prototype._didClickClose = function() {
  this._didCancel();
};

prototype._didClickDelete = function() {
  this.dispatchEvent(new Event(DELETE));
};

prototype.inProgress = function() {
  this._descriptionNode.empty();
  this._progressNode.html(this._inProgressString);
};

/**
 * @method showError
 * @param {String} error
 */
prototype.showError = function(error) {
  this._showError(error);
};

prototype._showError = function(error) {
  this._ticker.setText(error);
  this._ticker.show();
  this.setButtons(this._errorButtons);
};

prototype.showActivityIndicator = function() {
  if (this._inProgress) return;

  this._inProgress = true;
  this.disableButtons();
  this.hideCloseButton();
  this._descriptionNode.append(this._loadingIcon);
};

prototype.hideActivityIndicator = function() {
  if (!this._inProgress) return;

  this._inProgress = false;
  this._loadingIcon.detach();
  this.enableButtons();
  this.showCloseButton();
};

}());
