/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule filter-setting-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;
var ErrorDialog = dixim.ctt.ErrorDialog;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var FilterSettingDialogView;
var prototype;
var superConstructor;

/**
 * @class FilterSettingDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
FilterSettingDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(FilterSettingDialogView, ModalDialogView);
dixim.ctt.ad.FilterSettingDialogView = FilterSettingDialogView;
prototype = FilterSettingDialogView.prototype;
superConstructor = FilterSettingDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-setting-view-wrapper" />';
var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

var TEMPLATE_BODY_CONTENT = ['<div id="setting-filter">',
                             '<table id="filter-setting"></table>',
                             '<div id="coupling_conditions_bar" class="table_header"></div>',
                             '<div id="coupling_conditions"></div>',
                             '</div>'
                             ].join('');

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the submit button was clicked.
 *
 * @event submit
 */
var SUBMIT = 'submit';

/**
 * Triggered when the alldelete button was clicked.
 *
 * @event alldelete
 */
var ALLDELETE = 'alldelete';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;

  buttons = [
             {
               'text': _('ctt_dialog_button_label_close'),
               click: $.proxy(this._didClickClose, this)
             }
  ];

  configs  = configs || {};

  configs.title = configs.title || _('ad_dialog_filter_setting_dialog_title');
  configs.contentBox = TEMPLATE_CONTENT_BOX;
  configs.bodyContent = TEMPLATE_BODY_CONTENT;
  configs.buttons = buttons;
  configs.width = 600;
  configs.height = 'auto';

  this._model = new AutoDownloadApiClient();
  this._loadingIcon = $(TEMPLATE_LOADING);

  superConstructor.prototype._initialize.call(this, configs);
};

//check date YYYY/MM/DD
prototype.checkDate = function(value) {
  var chk_year;
  var chk_month;
  var chk_day;
  var chk_dt;

  if (value.match(/^\d{4}\/\d{2}\/\d{2}$/)) {
    chk_year = parseInt(value.substr(0, 4), 10);
    chk_month = parseInt(value.substr(5, 2), 10);
    chk_day = parseInt(value.substr(8,2), 10);
    if (chk_month >= 1 && chk_month <= 12 && chk_day >= 1 && chk_day <=31) {
      chk_dt = new Date(chk_year, chk_month-1, chk_day);
      return (chk_dt.getFullYear() === chk_year && chk_dt.getMonth()+1 === chk_month && chk_dt.getDate() === chk_day);
    }
  }
  return false;
};
//   check HH:MM
prototype.checkHourMinute = function(value) {
  var chk_array = value.split(":");
  var chk_hour = chk_array[0];
  var chk_minute = chk_array[1];

  return (
      value.match(/^\d{1,2}\:\d{1,2}$/) &&
      !isNaN(chk_hour) &&
      !isNaN(chk_minute) &&
      chk_hour >= 0 &&
      chk_hour <= 23 &&
      chk_minute >= 0 &&
      chk_minute <= 59);
};
//check MM
prototype.checkMinute = function(value) {
  return (
      value.match(/^\d{1,2}$/) &&
      !isNaN(value) &&
      value >= 0 &&
      value <= 59);
};

prototype._didCreateFilterSettingTable = function() {
  $('#filter-setting').html(
    [
       '<tr><td>',
       '<span id="filter_name">',_("ad_dialog_filter_setting_filter_name"),':','</span>',
       '</td><td>',
       '<input type="text" id="input_filter_name">',
       '</td><td>',
       '<div id="setting_button" class="div-btn right_setting_button">', '<a href="#">', _("ad_dialog_filter_setting_name"), '</a>', '</div>',
       '</td></tr>',
       '<tr><td>',
       '<span id="span_coupling_conditions">',_("ad_dialog_filter_setting_method_of_coupling_conditions"),':','</span>',
       '</td><td colspan="2">',
       '<input type="radio" id="radio_and_attribute" name="coupling_condition" checked>',
       '<span id="and_aggrement">',_("ad_dialog_filter_setting_and_agreement"),'</span>',
       '</td></tr>',
       '<tr><td>',
       '</td><td colspan="2">',
       '<input type="radio" id="radio_or_attribute" name="coupling_condition">',
       '<span id="or_aggrement">',_("ad_dialog_filter_setting_or_agreement"),'</span>',
       '</td></tr>',
       '<tr><td colspan="2">',
       '</td><td>',
       '<div id="all_delete_button" class="div-btn">', '<a href="#">', _("ad_dialog_filter_setting_all_delete_conditions"), '</a>', '</div>',
       '</td></tr>'
    ].join('')
  );

  $('#setting_button').on('click', dixim.proxy(function() {
    this.dispatchEvent(new Event(SUBMIT));
  }, this));

  $('#all_delete_button').on('click', dixim.proxy(function() {
    this.dispatchEvent(new Event(ALLDELETE));
  }, this));
};

prototype._didAddOperatorOption = function(query, selector) {
  var value;
  var text;

  for (var option in query) {
    if (query[option]) {
      switch(option) {
      case 'include':
        value = 1;
        text = _("ad_dialog_filter_setting_include");
        break;
      case 'not_include':
        value = 2;
        text = _("ad_dialog_filter_setting_does_not_include");
        break;
      case 'beginning':
        value = 3;
        text = _("ad_dialog_filter_setting_beginning_with");
        break;
      case 'ending':
        value = 5;
        text = _("ad_dialog_filter_setting_ending_with");
        break;
      case 'consistent':
        value = 7;
        text = _("ad_dialog_filter_setting_consistent");
        break;
      case 'not_consistent':
        value = 8;
        text = _("ad_dialog_filter_setting_does_not_consistent");
        break;
      case 'before':
        value = 10;
        text = _("ad_dialog_filter_setting_before");
        break;
      case 'after':
        value = 12;
        text = _("ad_dialog_filter_setting_after");
        break;
      case 'delete_item':
        value = "del";
        text = _("ad_dialog_filter_setting_delete_item");
        break;
      default:
        break;
      }
      $(selector).append(['<option value="',value,'">',text,'</option>'].join(''));
    }
  }
};

prototype._didAddConditionRow = function(name, row_num) {
  $(['#table_',name].join('')).append(
    [
     '<tr id="',name,'_tr',row_num,'"><td id="',name,'_td',row_num,'">',
     '<input type="text" id="input_',name,'_text_',row_num,'">',
     '</td><td id="operator_td">',
     '<select></select>',
     '</td></tr>'
     ].join('')
  );

  $(['#table_',name,' select'].join('')).on('change', function() {
    if ($(this).val() === "del") {
      $(this).parent('td').prev().find('input').val('').trigger('change');
      if ($(this).parent('td').parent('tr').prev().length !== 0 || $(this).parent('td').parent('tr').next().length > 0) {
        $(this).parent('td').parent('tr').remove();
      }
    }
  });

  $(['#input_',name,'_text_',row_num].join('')).on('change',function() {
    if ($(this).parent('td').next().find('select').val() === "del") {
      $(this).val('');
    } else if (name === "date" && !prototype.checkDate($(this).val()) && $(this).val() !== ''){
      prototype._showErrorDialog("error_dialog_mistake_date");
      $(this).val('');
    }
  });
  $(['#input_',name,'_text_',row_num].join('')).on('keyup change',function() {
    var query = {};
    var exist_flag = false;

    if ($(this).parent('td').next().find('select').val() === "del") {
      $(this).val('');
    }


    $(['#table_',name,' input[type="text"]'].join('')).each(function() {
      if ($(this).val() !== "") {
        exist_flag = true;
      }
    });

    if (exist_flag) {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    }

    if ($(this).val() === "") return;

    if ($(this).parent("td").parent("tr").next().length === 0) {
      prototype._didAddConditionRow(name,row_num+1);
      switch(name) {
      case 'title':
        query = {
          include: true,
          not_include: true,
          beginning: true,
          ending: true,
          consistent: true,
          not_consistent: true,
          delete_item: true
         };
        break;
      case 'channel':
        query = {
            include: true,
            not_include: true,
            consistent: true,
            not_consistent: true,
            delete_item: true
         };
        break;
      case 'description':
        query = {
          include: true,
          not_include: true,
          delete_item: true
        };
        break;
      case 'date':
        query = {
          consistent: true,
          not_consistent: true,
          before: true,
          after: true,
          delete_item: true
         };
        $('[id*="input_date_text"]').datepicker({
          dateFormat: "yy/mm/dd",
          changeMonth: true,
          changeYear: true
         });
        break;
      default:
        break;
      }
      prototype._didAddOperatorOption(query, $(['#table_',name,' select:last'].join('')));
    }
  });
};

prototype._didCreateGenreConditionTable = function(name) {
  var table_name = ['#table_',name].join('');
  var genre = [
              _("ad_dialog_filter_setting_genre_1"),
              _("ad_dialog_filter_setting_genre_2"),
              _("ad_dialog_filter_setting_genre_4"),
              _("ad_dialog_filter_setting_genre_8"),
              _("ad_dialog_filter_setting_genre_16"),
              _("ad_dialog_filter_setting_genre_32"),
              _("ad_dialog_filter_setting_genre_64"),
              _("ad_dialog_filter_setting_genre_128"),
              _("ad_dialog_filter_setting_genre_256"),
              _("ad_dialog_filter_setting_genre_512"),
              _("ad_dialog_filter_setting_genre_1024"),
              _("ad_dialog_filter_setting_genre_2048"),
              _("ad_dialog_filter_setting_genre_4096")
              ];

  $(table_name).append('<tr></tr>');
  for (var i = 0; i < 13; i += 1) {
    if ((i % 5) === 0) {
        $([table_name,' tr:last'].join('')).after('</tr><tr>');
    }
    $([table_name,' tr:last'].join('')).append(
      [
        '<td>',
        '<input type="checkbox" id="genre',i,'" name="genre_checkbox" value="',Math.pow(2,i),'">',
        '</td>',
        '<td>',
        '<span id="genre_str',i,'">',
        genre[i],
        '</span>',
        '</td>'
     ].join('')
    );
  }
  $([table_name,' input[type="checkbox"]'].join('')).on("click", function() {
    if ($([table_name,' input[type="checkbox"]:checked'].join('')).length === 0) {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    }
  });
};


prototype._didCreateDayOfTheWeekConditionTable = function(name) {
  var table_name = ['#table_',name].join('');
  var week = [
              _("ad_dialog_filter_setting_day_of_the_week_array_sunday"),
              _("ad_dialog_filter_setting_day_of_the_week_array_monday"),
              _("ad_dialog_filter_setting_day_of_the_week_array_tuesday"),
              _("ad_dialog_filter_setting_day_of_the_week_array_wednesday"),
              _("ad_dialog_filter_setting_day_of_the_week_array_thursday"),
              _("ad_dialog_filter_setting_day_of_the_week_array_friday"),
              _("ad_dialog_filter_setting_day_of_the_week_array_saturday")
              ];

  $(table_name).append('<tr></tr>');
  for (var i = 0; i < 7; i += 1) {
    $([table_name,' tr:last'].join('')).append(
      [
        '<td>',
        '<input type="checkbox" id="day_of_the_week',i,'" name="day_of_the_week_checkbox" value="',Math.pow(2,i),'">',
        '</td>'
     ].join('')
    );
  }
  $(table_name).append('<tr></tr>');
  for (var j = 0; j < 7; j += 1) {
    $([table_name,' tr:last'].join('')).append(
      [
        '<td>',
        '<span id="day_of_the_week_str',j,'">',
        week[j],
        '</span>',
        '</td>'
     ].join('')
    );
  }
  $([table_name,' input[type="checkbox"]'].join('')).on("click", function() {
    if ($([table_name,' input[type="checkbox"]:checked'].join('')).length === 0) {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    }
  });
};

prototype._didCreateDurationConditionTable = function(name) {
  var table_name = ['#table_',name].join('');
  var checkExist = function() {
    var exist = false;

    $([table_name,' [type="text"]'].join('')).each(function() {
      var id = $(this).prop('id');

      if ((id === 'input_longer' || id === 'input_shorter') && $(this).val() !== '') {
        exist = true;
      } else if ((id === 'input_equal' || id === 'input_error') && $('#input_equal').val() !== '' && $('#input_error').val() !== '') {
        exist = true;
       }
    });

    if (exist) {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    }
  };

  $(table_name).append(
  [
   '<tr><td>',
   '<input type="checkbox" id="checkbox_longer">',
   '<input type="text" id="input_longer" disabled>',
   '<span id="span_longer">',_("ad_dialog_filter_setting_longer"),'</span>',
   '</td></tr>',
   '<tr><td>',
   '<input type="checkbox" id="checkbox_shorter">',
   '<input type="text" id="input_shorter" disabled>',
   '<span id="span_shorter">',_("ad_dialog_filter_setting_shorter"),'</span>',
   '</td></tr>',
   '<tr><td>',
   '<input type="checkbox" id="checkbox_equal">',
   '<input type="text" id="input_equal" disabled>',
   '<span id="span_equal">',_("ad_dialog_filter_setting_equal"),'</span>',
   '</td><td>',
   '<span id="span_minute" class="error_right">',_("ad_dialog_filter_setting_minute"),'</span>',
   '<input type="text" id="input_error" class="error_right" disabled>',
   '<span id="span_error" class="error_right">',_("ad_dialog_filter_setting_a_error"),'</span>',
   '</td></tr>'
   ].join('')
  );

  $([table_name,' [type="checkbox"]'].join('')).on('change', function() {
    if ($(this).prop('id') === "checkbox_equal") {
      if ($(this).is(':checked')) {
        $(this).next().prop('disabled', false);
        if ($(this).next().val() !== "") {
          $(this).parent('td').next().find('input').prop('disabled', false);
         }
      } else {
        $(this).next().val('').prop('disabled', true);
        $(this).parent('td').next().find('input').val('').prop('disabled', true);
      }
      checkExist();
      return;
    }
    if ($(this).is(':checked')) {
      $(this).next().prop('disabled', false);
    } else {
      $(this).next().val('').prop('disabled', true);
    }
    checkExist();
  });

  $([table_name,' [type="text"]'].join('')).on('change', function() {
    if ($(this).prop('id') === "input_error") {
      if (!prototype.checkMinute($(this).val()) && $(this).val() !== '') {
        prototype._showErrorDialog("error_dialog_mistake_mm_duration");
         $(this).val('');
       }
      checkExist();
      return;
    }
    if (!prototype.checkHourMinute($(this).val()) && $(this).val() !== '') {
      prototype._showErrorDialog("error_dialog_mistake_hh_mm_duration");
      $(this).val('');
    } else {
      if ($(this).prop('id') === "input_equal") {
         $(this).parent('td').next().find('input').prop('disabled', false);
      }
    }
    checkExist();
  });
};

prototype._didCreateRecordingConditionTable = function(name) {
  var table_name = ['#table_',name].join('');
  var name_str = name.split("_")[1];
  var checkExist = function() {
    if ($(['#input_',name_str,'_time'].join('')).val() !== '' && $(['#input_',name_str,'_time_error'].join('')).val() !== '') {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    }
  };

  $(table_name).append(
    [
      '<tr><td>',
      '<input type="checkbox" id="checkbox_',name_str,'_time">',
      '<input type="text" id="input_',name_str,'_time" disabled>',
      '</td><td>',
      '<span id="span_',name_str,'_time_minute" class="error_right">',_("ad_dialog_filter_setting_minute"),'</span>',
      '<input type="text" id="input_',name_str,'_time_error" class="error_right" disabled>',
      '<span id="span_',name_str,'_time_error" class="error_right">',_("ad_dialog_filter_setting_a_error"),'</span>',
      '</td></tr>'
    ].join('')
  );

  $([table_name,' [type="checkbox"]'].join('')).on('change', function() {
    var input_text = $(this).next();
    var error_text = $(this).parent('td').next().find('input');

    if ($(this).is(':checked')) {
      $(input_text).prop('disabled', false);
      if ($(input_text).val() !== '') {
        $(error_text).prop('disabled', false);
      }
    } else {
      $(input_text).val('').prop('disabled', true);
      $(error_text).val('').prop('disabled', true);
    }
    checkExist();
  });

  $([table_name,' #input_',name_str,'_time'].join('')).on('change', function() {
    var error_text = $(this).parent('td').next().find('input');

    if (!prototype.checkHourMinute($(this).val()) && $(this).val() !== '') {
      prototype._showErrorDialog("error_dialog_mistake_hh_mm_duration");
      $(this).val('');
      checkExist();
      return;
    }
    $(error_text).prop('disabled', false);
    checkExist();
  });

  $([table_name,' #input_',name_str,'_time_error'].join('')).on('change', function() {
    if (!prototype.checkMinute($(this).val()) && $(this).val() !== '') {
      prototype._showErrorDialog("error_dialog_mistake_mm_duration");
      $(this).val('');
    }
    checkExist();
  });
};

prototype._didCreateBroadcastWaveConditionTable = function(name) {
  var table_name = ['#table_',name].join('');
  var broadcast = {dtb: 1, bs: 2, cs: 4, sptv: 8, catv: 16, iptv: 32};

  $(table_name).append('<tr></tr>');
  for (var str in broadcast) {
    $([table_name, ' tr'].join('')).append(
        [
          '<td>',
          '<input type="checkbox" id="',str,'_checkbox" value="',broadcast[str],'">',
          '</td><td>',
          '<span id="',str,'_span">',_(['ad_dialog_filter_setting_broadcast_',str].join('')),'</span>',
          '</td>'
        ].join('')
      );
  }

  $([table_name,' [type="checkbox"]'].join('')).on('change', function() {
    var exist = false;

    $([table_name, ' [type="checkbox"]'].join("")).each(function() {
      if ($(this).is(':checked')) {
        exist = true;
      }
    });
    if (exist) {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    }
  });
};

prototype._didCreateCopyCountConditionTable = function(name) {
  var table_name = ['#table_',name].join('');
  var count_option = "";

  for (var i = 0; i < 10; i += 1) {
    count_option = [count_option,'<option value="',i,'">',i+1,'</option>'].join('');
  }
  count_option = [count_option,'<option value="',10,'">',_("ad_dialog_filter_setting_limitless"),'</option>'].join('');

  $(table_name).append(
    [
       '<tr><td>',
       '<input type="checkbox" id="checkbox_larger">',
       '</td><td>',
       '<select disabled>',count_option,'</select>',
       '</td><td>',
       '<select disabled>',
       '<option value="11">',_("ad_dialog_filter_setting_more_than"),'</option>',
       '<option value="12">',_("ad_dialog_filter_setting_or_more"),'</option>',
       '</select>',
       '</td></tr>',
       '<tr><td>',
       '<input type="checkbox" id="checkbox_shorter">',
       '</td><td>',
       '<select disabled>',count_option,'</select>',
       '</td><td>',
       '<select disabled>',
       '<option value="9">',_("ad_dialog_filter_setting_less_than"),'</option>',
       '<option value="10">',_("ad_dialog_filter_setting_or_less"),'</option>',
       '</select>',
       '</td></tr>',
       '<tr><td>',
       '<input type="checkbox" id="checkbox_same">',
       '</td><td>',
       '<select disabled>',count_option,'</select>',
       '</td><td>',
       '<span value="7">',_("ad_dialog_filter_setting_equal"),'</span>',
       '</td></tr>'
    ].join('')
  );

  $([table_name,' [type="checkbox"]'].join('')).on('change', function() {
    var exist = false;

    $(this).parent('td').parent('tr').find('select').prop('disabled', !$(this).is(':checked'));

    $([table_name, ' [type="checkbox"]'].join("")).each(function() {
      if ($(this).is(':checked')) {
        exist = true;
      }
    });
    if (exist) {
      $(['#',name,'_state_span span'].join('')).prop('id','exist_state').html(_("ad_dialog_filter_setting_state_exist"));
    } else {
      $(['#',name,'_state_span span'].join('')).prop('id','not_exist_state').html(_("ad_dialog_filter_setting_state_not_exist"));
    }
  });
};

prototype._didCreateConditionTable = function(name) {
  var query = {};

  $(['#dd_',name].join('')).html(
    [
      '<table id="table_',name,'">',
      '</table>'
    ].join('')
  );
  switch(name) {
  case 'title':
    this._didAddConditionRow(name, 1);
    query = {
        include: true,
        not_include: true,
        beginning: true,
        ending: true,
        consistent: true,
        not_consistent: true,
        delete_item: true
    };
    this._didAddOperatorOption(query, $(['#table_',name,' select'].join('')));
    break;
  case 'channel':
    this._didAddConditionRow(name, 1);
    query = {
        include: true,
        not_include: true,
        consistent: true,
        not_consistent: true,
        delete_item: true
    };
    this._didAddOperatorOption(query, $(['#table_',name,' select'].join('')));
    break;
  case 'description':
    this._didAddConditionRow(name, 1);
    query = {
      include: true,
      not_include: true,
      delete_item: true
    };
    this._didAddOperatorOption(query, $(['#table_',name,' select'].join('')));
    break;
  case 'date':
    this._didAddConditionRow(name, 1);
    query = {
      consistent: true,
      not_consistent: true,
      before: true,
      after: true,
      delete_item: true
    };
    $('[id*="input_date_text"]').datepicker({
      dateFormat: "yy/mm/dd",
      changeMonth: true,
      changeYear: true
    });
    this._didAddOperatorOption(query, $(['#table_',name,' select'].join('')));
    break;
  case 'genre':
    this._didCreateGenreConditionTable(name);
    break;
  case 'day_of_the_week':
    this._didCreateDayOfTheWeekConditionTable(name);
    break;
  case 'duration':
    this._didCreateDurationConditionTable(name);
    break;
  case 'recording_start_time':
    this._didCreateRecordingConditionTable(name);
    break;
  case 'recording_end_time':
    this._didCreateRecordingConditionTable(name);
    break;
  case 'broadcast_wave':
    this._didCreateBroadcastWaveConditionTable(name);
    break;
  case 'copy_count':
    this._didCreateCopyCountConditionTable(name);
    break;
  default:
    break;
  }
};

prototype._didAddCouplingConditionRow = function(name) {
  var condition_name = _("ad_dialog_filter_setting_" + name);

  $('#setting_coupling_condition').append(
  [
   '<dt id="dt_',name,'">',condition_name,
   '<span id="',name,'_state_span">','(',_("ad_dialog_filter_setting_state_of_conditions"),
   '<span id="not_exist_state">',_("ad_dialog_filter_setting_state_not_exist"),'</span>',
   ')','</span>',
   '</dt>',
   '<dd id="dd_',name,'">',name,'</dd>'
   ].join('')
  );
};

prototype._didCreateCouplingConditions = function() {
  var conditions = ["title",
                    "channel",
                    "genre",
                    "description",
                    "date",
                    "day_of_the_week",
                    "duration",
                    "recording_start_time",
                    "recording_end_time",
                    "broadcast_wave",
                    "copy_count"
                    ];
  var conditions_length = conditions.length;
  var condition;

  $('#coupling_conditions_bar').html(_("ad_dialog_filter_setting_condition"));
  $('#coupling_conditions').html('<dl id="setting_coupling_condition"></dl>');
  for (var i = 0; i < conditions_length; i += 1) {
    condition = conditions[i];
    this._didAddCouplingConditionRow(condition);
    this._didCreateConditionTable(condition);
  }

  $('#coupling_conditions dt').on('click',function() {
    $(this).next().slideToggle();
    $('#coupling_conditions dd').not($(this).next()).slideUp();
  });
};

prototype._didRenderDialog = function(e) {
  this._didCreateFilterSettingTable();
  this._didCreateCouplingConditions();

  superConstructor.prototype._didRenderDialog.call(this, e);
  $('#ui-datepicker-div').css('border', '0px');
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickClose = function() {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
