/**
 * @module ctt
 * @submodule transfer-history-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var TransferHistoryView = dixim.ctt.TransferHistoryView;
var TransferHistoryListingController = dixim.ctt.TransferHistoryListingController;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var _ = dixim.ctt.ResourceBundle.get;

var TransferHistoryController;
var prototype;

/**
 * @class TransferHistoryController
 * @constructor
 * @param {Object} configs
 */
TransferHistoryController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(TransferHistoryController, EventTarget);
dixim.ctt.TransferHistoryController = TransferHistoryController;
prototype = TransferHistoryController.prototype;

/**
 * Triggered when the status dialog has been closed.
 * 
 * @event close
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = configs.model || new ContentsTransferApiClient();
  this._historyListingController = null;
  this._view = new TransferHistoryView();
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('close', this._didClickClose, this);
  v.addEventListener('clear', this._didClickClear, this);
  v.addEventListener('render', this._didRenderView, this);

  m.addEventListener('logList', this._didFetchLogList, this);
  m.addEventListener('logListError', this._didFailLogList, this);
  m.addEventListener('clearLogList', this._didClearLogList, this);
  m.addEventListener('clearLogListError', this._didFailClearLogList, this);

  v.show();

  if (!!this._historyListingController) {
    this._historyListingController.start(); 
  }
};

prototype._didRenderView = function(e) {
  var lc;

  lc = new TransferHistoryListingController({
    containerNode: this._view.getBodyContent()
  });
  lc.start();

  this._view.showActivityIndicator();
  this._historyListingController = lc;
  this._fetchLogList();
};

prototype._fetchLogList = function(e) {
  if (!!this._session) return;
  this._session = this._model.logList();
};

prototype._didClickClose = function(e) {
  this._stop();
  this.dispatchEvent(new Event(CLOSE));
};

prototype._didClickClear = function(e) {
  if (!!this._session) return;

  this._view.showActivityIndicator();
  this._session = this._model.clearLogList();
};

prototype._didFetchLogList = function(e) {
  this._session = null;
  this._historyListingController.update(e.args.data.results);
  this._view.hideActivityIndicator();
};

prototype._didClearLogList = function(e) {
  this._session = null;
  this._fetchLogList();
};

prototype._didFailLogList = function(e) {
  this._session = null;
  this._view.showError(_('ctt_transfer_history_view_error_fetch_log_list'));
  this._view.hideActivityIndicator();
};

prototype._didFailClearLogList = function(e) {
  this._session = null;
  this._view.showError(_('ctt_transfer_history_view_error_clear_log_list'));
  this._view.hideActivityIndicator();
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._stop();
};

prototype._stop = function() {
  if (!!this._historyListingController) {
    this._historyListingController.stop();
  }
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  if (!!this._historyListingController) {
    this._historyListingController.destroy();
    this._historyListingController = null;
  }
  this._view.destroy();
};

prototype.setPin = function(pin) {
  this._model.setPin(pin);
};

}());
