/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule api-client-session
 */
(function() {
var ApiClientSession;
var prototype;

/**
 * Represents a session data of API action.
 *
 * @class ApiClientSession
 * @param {String} api API action name. (such as 'transfer_status', 'transfer', etc.)
 * @param {Object} query Query data used for the action.
 * @param {Object} [userData=null] Use-specific data.
 */
ApiClientSession = function(api, query, userData) {
    /**
    * @property api
    * @type {String} API name
    */
    this.api = api;

    /**
    * @property query
    * @type {Object} query data used for the request.
    */
    this.query = query;

    /**
    * @property userData
    * @type {Object} user-specific data;
    */
    this.userData = userData || null;

    /**
    * Request object.
    *
    * Some APIs don't set this value,
    * so please do null-check before accessing this property.
    *
    * @property request
    * @type {Object}
    */
    this.request = null;

    /**
    * @property connection
    * @type {jQuery}
    */
    this.connection = null;

    /**
    * @property _isCancelled
    * @private
    * @type {boolean}
    */
    this._isCancelled = false;
};
dixim.common.ApiClientSession = ApiClientSession;
prototype = ApiClientSession.prototype;

/**
 * Returns whether the session is cancelled or not.
 *
 * @method isCancelled
 * @return {boolean}
 */
prototype.isCancelled = function() {
    return this._isCancelled;
};

/**
 * Cancels session.
 *
 * Currently, it just make the session cancelled state.
 *
 * @method cancel
 */
prototype.cancel = function () {
    this._isCancelled = true;
};

}());
