/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule server-boot-view
 */

(function() {
var ServerBootView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var $ = dixim.ctt.jQuery;

/**
 * @class ServerBootView
 * @constructor
 * @param {Object} configs
 */
ServerBootView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ServerBootView, EventTarget);
dixim.server.ServerBootView = ServerBootView;
prototype = ServerBootView.prototype;
superConstructor = ServerBootView.superConstructor;

var PREV = 'prev';
var STATUS = 'status';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._title = _('server_menu_boot_server');
};

/**
 * @method show
 */
prototype.show = function() {
    $('.rootPage').html([
        '<div class="contents" data-enhance="false">',
        '<h2 id="tit_page">', this._title, '</h2>',
        '<div class="contents_inner">',
        '<div class="dl_tbl">',
        '<dl>',
        '<dt id="status_title">', '</dt>',
        '<dd>',
        '<input id="status_val" type="text" readonly="readonly" />',
        '<input id="status_btn" type="button" class="btn color_btn" readonly="readonly" />',
        '</dd>',
        '</dl>',
        '</div>',
        '</div>',
        '</div>',
    ].join(''));

    $('.art_tit').text(this._title);
    $('.btn_prev').show().one('click', dixim.proxy(this._clickPrev, this));
    $('#status_btn').on('click', dixim.proxy(this._clickStatus, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
    $('.rootPage').empty();
};

prototype.setValue = function(id, val, operation) {
    var selector = $(['#', id].join(''));

    if (operation == 'text') {
        selector.text(val);
    } else if (operation == 'val') {
        selector.val(val);
    }
};

prototype.buttonStatus = function(id, is_disable) {
    var selector = $(['#', id].join(''));

    if (is_disable === null) {
        selector.hide();
        return;
    }
    selector.show().prop('disabled', is_disable);
};

prototype._clickPrev = function() {
    this.dispatchEvent(new Event(PREV));
};

prototype._clickStatus = function() {
    this.dispatchEvent(new Event(STATUS));
};

}());
