# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command

from ionas.request import Entry, XmlNode
from fal_info import Results, FAL_DIXIMDMS_PATH

def _xmlnode__is_true(xmlnode, xpath='.', default=bool(0)):
    try:
        return xmlnode.get_value(xpath) == 'true'
    except:
        return default

class FALDiximDMS(Command):

    COMMAND = FAL_DIXIMDMS_PATH

    def load(self):
        def __dict(entry):
            d = {}
            try:
                diximdms_node = entry.get()
                try:
                    d["enable"] = diximdms_node.get_node("service").is_set_to_true("enable")
                    d["language"] = diximdms_node.get_value("lang")
                    d["sharelist"] = []
                    for share_node in diximdms_node.get_node_list("sharelist/share"):
                        d["sharelist"].append({
                            "name": share_node.get_value("name"),
                        })
                except:
                    d["enable"] = False
                    d["language"] = "ja"
                    d["sharelist"] = []

            except:
                pass

            return d

        try:
            entry = self._load()

            return __dict(entry)
        except:
            return {}

    def query(self):
        def __dict(entry):
            d = {}
            try:
                diximdms_node = entry.get()
                try:
                    d["status"] = diximdms_node.get_value("statusinfo/status")
                    d["lasttime"] = {
                        "timestamp": diximdms_node.get_value("statusinfo/lasttime/timestamp"),
                    }
                    d["needreboot"] = _xmlnode__is_true(diximdms_node, "dtcp/@needreboot", bool(1))
                    d["licensekey"] = diximdms_node.get_value("dtcp/license/key")
                except:
                    d["status"] = None
                    d["lasttime"] = {
                        "timestamp": None,
                    }
                    d["needreboot"] = True
                    d["licensekey"] = ""

            except:
                pass

            return d

        try:
            entry = self._query()
            return __dict(entry)
        except:
            return {}

    def apply(self, data=None, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_DIXIMDMS_CHANGE", {}],
            None: ["ERR_DIXIMDMS_CHANGE", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._apply(data=data)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def initialize(self, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_DIXIMDMS_INITIALIZE_START", {}],
            None: ["ERR_DIXIMDMS_INITIALIZE_START", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._initialize()
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def register_licensekey(self, data=None, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_DIXIMDMS_DTCP_UPGRADE", {}],
            Results.INVALID_DATA: ["ERR_DIXIMDMS_INVALID_LICENSE_KEY", {}],
            Results.CONNECT_ERROR: ["ERR_DIXIMDMS_SERVER_CONNECT", {}],
            None: ["ERR_DIXIMDMS_DTCP_UPGRADE", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._register_licensekey(data=data)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def delete_reservation(self, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_DIXIMDMS_DELETE_XSRS_INFO", {}],
            None: ["ERR_DIXIMDMS_XSRS_INFO", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._delete_reservation()
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _load(self):
        entry = Entry("load", """<diximdms/>""")
        return self.request(entry)

    def _query(self):
        entry = Entry("query", """<diximdms/>""")
        return self.request(entry)

    def _apply(self, data=None):
        entry = Entry("apply", """<diximdms/>""")
        diximdms_node = entry.get()

        try:
            if "enable" in data:
                diximdms_node.append_child("service")
                service_node = diximdms_node.get_node("service")
                service_node.set_bool_attr("enable", data["enable"])

            if "language" in data:
                diximdms_node.append_child("lang", text=str(data["language"]))

            diximdms_node.append_child("sharelist")
            sharelist_node = diximdms_node.get_node("sharelist")
            for share in data["sharelist"]:
                share_node = XmlNode("<share/>")
                share_node.append_child("name", text=share["name"])
                sharelist_node.append_child(share_node)
        except:
            pass

        return self.request(entry)

    def _initialize(self):
        entry = Entry("initialize", """<diximdms/>""")
        return self.request(entry)

    def _delete_reservation(self):
        entry = Entry("delete", """<diximdms/>""")
        return self.request(entry)

    def _register_licensekey(self, data=None):
        try:
            entry = Entry("register", """<diximdms/>""")
            diximdms_node = entry.get()
            dtcp_info = XmlNode("<dtcp/>")
            lisence_info = XmlNode("<license/>")
            lisence_info.append_child("key", text=str(data["key"]))
            dtcp_info.append_child(lisence_info)
            diximdms_node.append_child(dtcp_info)
        except:
            pass

        return self.request(entry)

if __name__ == u"__main__":
    fal = FALDiximDMS()
